#!/usr/bin/env python
#
# This single line is required for "pkgutil-style namespace package"
# DO NOT EDIT this file
#######################
__path__ = __import__('pkgutil').extend_path(__path__, __name__)