#!/usr/bin/env python

__author__ = "Padraic Shafer"
__copyright__ = "Copyright (c) 2022, Padraic Shafer"
__credits__ = [__author__, ]
__license__ = ""
__maintainer__ = "Padraic Shafer"
__email__ = "PShafer@lbl.gov"
__status__ = "Development"

from .version import __version__, __date__, __credits__
